"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  MySQLSessionStorage: () => MySQLSessionStorage
});
module.exports = __toCommonJS(src_exports);

// src/mysql.ts
var import_shopify_api = require("@shopify/shopify-api");

// src/migrations.ts
var import_shopify_app_session_storage = require("@shopify/shopify-app-session-storage");
var migrationList = [
  new import_shopify_app_session_storage.MigrationOperation(
    "migrateScopeFieldToVarchar1024",
    migrateScopeFieldToVarchar1024
  )
];
async function migrateScopeFieldToVarchar1024(connection) {
  await connection.query(`ALTER TABLE ${connection.sessionStorageIdentifier} 
      MODIFY COLUMN scope varchar(1024)`);
}

// src/mysql-connection.ts
var mysql = __toESM(require("mysql2/promise"));
var MySqlConnection = class {
  sessionStorageIdentifier;
  ready;
  connection;
  constructor(sql_config, sessionStorageIdentifier) {
    this.ready = this.init(sql_config);
    this.sessionStorageIdentifier = sessionStorageIdentifier;
  }
  async query(query, params = []) {
    await this.ready;
    return this.connection.query(query, params);
  }
  async connect() {
    await this.ready;
    return Promise.resolve();
  }
  async disconnect() {
    await this.ready;
    await this.connection.end();
  }
  getDatabase() {
    return this.connection.config.database;
  }
  async hasTable(tablename) {
    await this.ready;
    const query = `
      SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES 
        WHERE TABLE_NAME = ${this.getArgumentPlaceholder()} 
          AND TABLE_SCHEMA = ${this.getArgumentPlaceholder()};
    `;
    const [rows] = await this.connection.query(query, [
      tablename,
      this.getDatabase()
    ]);
    return Array.isArray(rows) && rows.length === 1;
  }
  getArgumentPlaceholder(_) {
    return `?`;
  }
  async init(sql_config) {
    this.connection = await mysql.createConnection({
      user: sql_config.username,
      password: sql_config.password,
      host: sql_config.host,
      database: sql_config.dbName
    });
  }
};

// src/mysql-migrator.ts
var import_shopify_app_session_storage2 = require("@shopify/shopify-app-session-storage");
var MySqlSessionStorageMigrator = class extends import_shopify_app_session_storage2.RdbmsSessionStorageMigrator {
  constructor(dbConnection, opts = {}) {
    super(dbConnection, opts);
  }
  async initMigrationPersistence() {
    const discardCreateTable = await this.connection.hasTable(
      this.options.migrationDBIdentifier
    );
    if (!discardCreateTable) {
      const migration = `
      CREATE TABLE ${this.options.migrationDBIdentifier} (
        ${this.getOptions().migrationNameColumnName} varchar(255) NOT NULL PRIMARY KEY
    );`;
      await this.connection.query(migration, []);
    }
  }
};

// src/mysql.ts
var defaultMySQLSessionStorageOptions = {
  sessionTableName: "shopify_sessions",
  migratorOptions: {
    migrationDBIdentifier: "shopify_sessions_migrations",
    migrationNameColumnName: "migration_name",
    migrations: migrationList
  }
};
var MySQLSessionStorage = class {
  static withCredentials(sql_config, opts) {
    return new MySQLSessionStorage(
      sql_config,
      opts
    );
  }
  ready;
  internalInit;
  options;
  connection;
  migrator;
  constructor(sql_config, opts = {}) {
    this.options = { ...defaultMySQLSessionStorageOptions, ...opts };
    this.internalInit = this.init(sql_config);
    this.ready = this.initMigrator(this.options.migratorOptions);
  }
  async storeSession(session) {
    await this.ready;
    const entries = session.toPropertyArray().map(
      ([key, value]) => key === "expires" ? [key, Math.floor(value / 1e3)] : [key, value]
    );
    const query = `
      REPLACE INTO ${this.options.sessionTableName}
      (${entries.map(([key]) => key).join(", ")})
      VALUES (${entries.map(() => `${this.connection.getArgumentPlaceholder()}`).join(", ")})
    `;
    await this.connection.query(
      query,
      entries.map(([_key, value]) => value)
    );
    return true;
  }
  async loadSession(id) {
    await this.ready;
    const query = `
      SELECT * FROM \`${this.options.sessionTableName}\`
      WHERE id = ${this.connection.getArgumentPlaceholder()};
    `;
    const [rows] = await this.connection.query(query, [id]);
    if (!Array.isArray(rows) || rows?.length !== 1)
      return void 0;
    const rawResult = rows[0];
    return this.databaseRowToSession(rawResult);
  }
  async deleteSession(id) {
    await this.ready;
    const query = `
      DELETE FROM ${this.options.sessionTableName}
      WHERE id = ${this.connection.getArgumentPlaceholder()};
    `;
    await this.connection.query(query, [id]);
    return true;
  }
  async deleteSessions(ids) {
    await this.ready;
    const query = `
      DELETE FROM ${this.options.sessionTableName}
      WHERE id IN (${ids.map(() => `${this.connection.getArgumentPlaceholder()}`).join(",")});
    `;
    await this.connection.query(query, ids);
    return true;
  }
  async findSessionsByShop(shop) {
    await this.ready;
    const query = `
      SELECT * FROM ${this.options.sessionTableName}
      WHERE shop = ${this.connection.getArgumentPlaceholder()};
    `;
    const [rows] = await this.connection.query(query, [shop]);
    if (!Array.isArray(rows) || rows?.length === 0)
      return [];
    const results = rows.map((row) => {
      return this.databaseRowToSession(row);
    });
    return results;
  }
  async disconnect() {
    await this.connection.disconnect();
  }
  async init(sql_config) {
    this.connection = new MySqlConnection(sql_config, this.options.sessionTableName);
    await this.createTable();
  }
  async createTable() {
    const hasSessionTable = await this.connection.hasTable(
      this.options.sessionTableName
    );
    if (!hasSessionTable) {
      const query = `
        CREATE TABLE ${this.options.sessionTableName} (
          id varchar(255) NOT NULL PRIMARY KEY,
          shop varchar(255) NOT NULL,
          state varchar(255) NOT NULL,
          isOnline tinyint NOT NULL,
          scope varchar(255),
          expires integer,
          onlineAccessInfo varchar(255),
          accessToken varchar(255)
        )
      `;
      await this.connection.query(query);
    }
  }
  databaseRowToSession(row) {
    if (row.expires)
      row.expires *= 1e3;
    return import_shopify_api.Session.fromPropertyArray(Object.entries(row));
  }
  async initMigrator(migratorOptions) {
    await this.internalInit;
    if (migratorOptions === null) {
      return Promise.resolve();
    } else {
      this.migrator = new MySqlSessionStorageMigrator(
        this.connection,
        migratorOptions
      );
      this.migrator.validateMigrationList(migrationList);
      return this.migrator.applyMigrations();
    }
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  MySQLSessionStorage
});
//# sourceMappingURL=index.js.map
